import adbc_driver_flightsql.dbapi
import adbc_driver_manager
import pandas

from ._connection import KamuConnection


class KamuConnectionFlightSql(KamuConnection):
    """
    `KamuConnection` implementation using ADBC client protocol
    """

    def __init__(self, url):
        super().__init__()

        self._url = url

        self._adbc = adbc_driver_flightsql.dbapi.connect(
            self._url,
            db_kwargs={
                adbc_driver_manager.DatabaseOptions.USERNAME.value: "kamu",
                adbc_driver_manager.DatabaseOptions.PASSWORD.value: "kamu",
            },
            autocommit=True,
        )

    def __repr__(self):
        return f"KamuConnectionFlightSql(url='{self._url}')"

    def url(self):
        return self._url

    def query(self, sql):
        return pandas.read_sql(sql, self.as_adbc())

    def as_adbc(self):
        return self._adbc

    def __enter__(self):
        self._adbc.__enter__()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        return self._adbc.__exit__(exc_type, exc_value, traceback)

    def close(self):
        self._adbc.close()



import os

from ._connection import KamuConnection

__version__ = "0.4.1"


def connect(url=None, engine=None, connection_config=None) -> KamuConnection:
    """
    Open connection to a Kamu node.

    Examples
    --------
    >>> import kamu
    >>>
    >>> # Connect to secure node
    >>> with kamu.connect("grpc+tls://node.demo.kamu.dev:50050") as con:
    >>>     pass
    >>>
    >>> # Connect to local insecure node
    >>> with kamu.connect("grpc://localhost:50050") as con:
    >>>     pass
    """
    url = url or os.environ.get("KAMU_CLIENT_URL")
    if not url:
        raise ValueError("url is not specified")

    engine = (engine or "datafusion").lower()

    connection_config = connection_config or {}

    if engine == "datafusion":
        from . import _connection_flight_sql

        return _connection_flight_sql.KamuConnectionFlightSql(
            url=url, **connection_config
        )
    if engine == "spark":
        from . import _connection_livy

        return _connection_livy.KamuConnectionLivy(url=url, **connection_config)

    raise ValueError(f"Engine '{engine}' is not supported")


def load_ipython_extension(ipython):
    """
    Called when running `%load_ext kamu` in Jupyter / IPython.
    """
    from . import _jupyter

    ipython.register_magics(_jupyter.KamuMagics)

    try:
        import autovizwidget.widget.utils

        autoviz = autovizwidget.widget.utils.display_dataframe
    except ImportError:
        autoviz = None

    if autoviz:
        ipython.display_formatter.ipython_display_formatter.for_type_by_name(
            "pandas.core.frame",
            "DataFrame",
            autoviz,
        )



import re

import adbc_driver_manager
import pandas
import pytest

import kamu


def test_version():
    assert re.fullmatch(
        r"\d\.\d\.\d", kamu.__version__
    ), "Version doesn't match the pattern"


def test_repr(server_flightsql_mt):
    with kamu.connect(server_flightsql_mt.url) as con:
        assert repr(con) == f"KamuConnectionFlightSql(url='{server_flightsql_mt.url}')"


def test_sql_query_minimal(server_flightsql_mt):
    with kamu.connect(server_flightsql_mt.url) as con:
        actual = con.query("select 1 as value")
        expected = pandas.DataFrame({"value": [1]})
        pandas.testing.assert_frame_equal(expected, actual)


def test_use_after_close(server_flightsql_mt):
    con = kamu.connect(server_flightsql_mt.url)
    actual = con.query("select 1 as value")
    expected = pandas.DataFrame({"value": [1]})
    pandas.testing.assert_frame_equal(expected, actual)
    con.close()
    with pytest.raises(adbc_driver_manager.ProgrammingError):
        con.query("select 1 as value")


def test_query_pandas_interop(server_flightsql_mt):
    with kamu.connect(server_flightsql_mt.url) as con:
        actual = pandas.read_sql_query("select 1 as value", con.as_adbc())
        expected = pandas.DataFrame({"value": [1]})
        pandas.testing.assert_frame_equal(expected, actual)


def test_query(server_flightsql_mt):
    with kamu.connect(server_flightsql_mt.url) as con:
        actual = con.query(
            """
            select
                offset,
                op,
                reported_date,
                id,
                gender,
                age_group,
                location
            from 'kamu/covid19.british-columbia.case-details.hm'
            order by offset
            limit 1
            """
        )

        expected = pandas.DataFrame(
            {
                "offset": [0],
                "op": [0],
                "reported_date": ["2020-01-29T00:00:00.000Z"],
                "id": [1],
                "gender": ["M"],
                "age_group": ["40s"],
                "location": ["Out of Canada"],
            }
        ).astype(
            dtype={
                "offset": "int64",
                "op": "int32",
                "reported_date": "datetime64[ms, UTC]",
                "id": "int64",
            }
        )

        pandas.testing.assert_frame_equal(expected, actual)
